<?php

declare(strict_types=1);

namespace Dotclear\Plugin\HideStoreUpdates;

use ArrayObject;
use Dotclear\App;
use Dotclear\Core\Backend\ModulesList;
use Dotclear\Core\Backend\Notices;
use Dotclear\Database\MetaRecord;
use Dotclear\Helper\Html\Form\Submit;
use Dotclear\Helper\Html\Html;
use Dotclear\Module\ModuleDefine;

/**
 * @brief       HideStoreUpdates backend behaviors class.
 * @ingroup     HideStoreUpdates
 *
 * @author      Jean-Christian Denis
 * @copyright   GPL-2.0 https://www.gnu.org/licenses/gpl-2.0.html
 */
class BackendBehaviors
{
    private static bool $cleared = false;

    public static function adminModulesListGetActionsV2(ModulesList $list, ModuleDefine $define): string
    {
        return in_array($list->getList(), [
            'plugin-update',
            'theme-update',
        ]) ? (new Submit([My::id() . '[' . Html::escapeHTML($define->getId()) . ' ' . $define->get('version') . ']']))->value(__('Hide'))->render() : '';
    }

    /**
     * @param   array<int, ModuleDefine>    $defines    The modules
     */
    public static function adminModulesListDoActions(ModulesList $list, array $defines, string $type): void
    {
        // Hide an update version
        if (empty($_POST[My::id()])
         || !is_array($_POST[My::id()])) {
            return;
        }

        $type   = str_starts_with($type, 'theme') ? 'themes' : 'plugins';
        $to_add = key($_POST[My::id()]);

        $logs = self::getLogs($type);
        if (!$logs->isEmpty()) {
            while ($logs->fetch()) {
                if ($to_add == $logs->f('log_msg')) {
                    // Allready hidden!
                    return;
                }
            }
        }
        if (!empty($to_add)) {
            $cur = App::log()->openLogCursor();
            $cur->setField('log_table', My::id() . '_' . $type);
            $cur->setField('log_msg', $to_add);
            App::log()->addLog($cur);

            Notices::AddSuccessNotice(__('Module successfully hidden.'));
        }
    }

    /**
     * @param   ArrayObject<int, ModuleDefine>  $defines    The modules
     */
    public static function afterCheckStoreUpdate(string $type, ArrayObject $defines): void
    {
        self::clearLogs($type);

        $logs = self::getLogs($type);
        if ($logs->isEmpty()) {
            return;
        }

        $hiddens = [];
        while ($logs->fetch()) {
            $hiddens[] = $logs->f('log_msg');
        }

        foreach ($defines as $k => $define) {
            if (in_array($define->getId() . ' ' . $define->get('version'), $hiddens)) {
                $defines->offsetUnset($k);
            }
        }
    }

    private static function clearLogs(string $type): void
    {
        if (!self::$cleared && !empty($_GET['nocache'])) {
            $logs = self::getLogs($type);
            if (!$logs->isEmpty()) {
                $ids = [];
                while ($logs->fetch()) {
                    $ids[] = (int) $logs->f('log_id');
                }
                App::log()->delLogs($ids);
            }
            self::$cleared = true;
        }
    }

    private static function getLogs(string $type): MetaRecord
    {
        return App::log()->getLogs([
            'log_table' => My::id() . '_' . $type,
            'blog_id'   => $type == 'plugins' ? '*' : App::blog()->id(),
        ]);
    }
}
